package org.vaadin.sami.autocomplete;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.vaadin.sami.autocomplete.AutoCompleteTextField.Suggestion;

import com.vaadin.Application;
import com.vaadin.ui.Label;
import com.vaadin.ui.Window;

@SuppressWarnings("serial")
public class AutoCompleteSampleApplication extends Application {

	@Override
	public void init() {
		Window w = new Window();
		addWindow(w);
		w.addComponent(new Label("Use CTRL-Space to complete the locale name."));
		AutoCompleteTextField acEditor = new AutoCompleteTextField();
		acEditor.setWidth("150px");
		acEditor.setSuggester(new SampleSuggester());
		w.addComponent(acEditor);
		acEditor.focus();
		acEditor.setImmediate(true);
	}

	public class SampleSuggester implements AutoCompleteTextField.Suggester {

		public List<Suggestion> getSuggestions(
				final AutoCompleteTextField source, final String text,
				final int cursorPosition) {

			// Parse the current word prefix out from the text
			String caption;
			if (text != null && cursorPosition <= text.length()) {
				// Find the prefix
				caption = text.substring(0, cursorPosition);
				// int lastWordStart = Math.max(prefix.lastIndexOf(' '),
				// prefix.lastIndexOf('\n'));
				int lastWordStart = -1;
				if (lastWordStart < cursorPosition) {
					caption = text.substring(lastWordStart + 1, cursorPosition);
				} else {
					caption = "";
				}
			} else {
				caption = "";
			}

			// Look up matching items
			caption = caption.toLowerCase();
			List<Suggestion> found = new ArrayList<Suggestion>();
			Locale[] l = Locale.getAvailableLocales();
			for (int i = 0; i < l.length; i++) {
				String name = l[i].getDisplayName();
				if (name.toLowerCase().contains(caption)) {

					// Value is the same as name without the part in parentheses
					String value = name;

					// Create a new suggestion
					found.add(source.createSuggestion(name, value, null, 0,
							cursorPosition));
				}
			}
			return found;
		}
	}

}
